@@ -1,4 +1,20 @@
-/*
+/*  Copyright 1999 Peter Schlaile.
+ *  Copyright 1999-2002,2006,2007,2009 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
  * Small install-test utility to check if a floppyd-server is running on the
  * X-Server-Host.
  *
@@ -13,9 +29,6 @@
 #include "sysincludes.h"
 #include "stream.h"
 #include "mtools.h"
-#include "msdos.h"
-#include "scsi.h"
-#include "partition.h"
 #include "floppyd_io.h"
 
 #ifdef USE_FLOPPYD
@@ -25,10 +38,7 @@
 
 /* ######################################################################## */
 
-typedef unsigned char Byte;
-typedef unsigned long Dword;
-
-const char* AuthErrors[] = {
+static const char* AuthErrors[] = {
 	"Auth success!",
 	"Auth failed: Packet oversized!",
 	"Auth failed: X-Cookie doesn't match!",
@@ -39,42 +49,46 @@ const char* AuthErrors[] = {
 #include "byte_dword.h"
 #include "read_dword.h"
 
-static void write_dword(int handle, Dword parm) 
+static int write_dword(int handle, Dword parm)
 {
 	Byte val[4];
 
 	dword2byte(parm, val);
 
-	write(handle, val, 4);
+	if(write(handle, val, 4) < 4)
+		return -1;
+	return 0;
 }
 
 
 /* ######################################################################## */
 
-static int authenticate_to_floppyd(char fullauth, int sock, char *display, 
-				   int protoversion)
+static uint32_t authenticate_to_floppyd(char fullauth, int sock, char *display,
+					uint32_t protoversion)
 {
-	off_t filelen=0;
+	uint32_t filelen=0;
 	Byte buf[16];
 	const char *command[] = { "xauth", "xauth", "extract", "-", 0, 0 };
 	char *xcookie = NULL;
 	Dword errcode;
-	int bytesRead;
+	uint32_t bytesRead;
+	uint32_t cap=0;
 
 	if (fullauth) {
 		command[4] = display;
 
-		filelen=strlen(display);
+		filelen=(uint32_t)strlen(display);
 		filelen += 100;
 
 		xcookie = (char *) safe_malloc(filelen+4);
-		filelen = safePopenOut(command, xcookie+4, filelen);
+		filelen = (uint32_t) safePopenOut(command, xcookie+4, filelen);
 		if(filelen < 1)
 		    return AUTH_AUTHFAILED;
 	}
 	dword2byte(4,buf);
 	dword2byte(protoversion,buf+4);
-	write(sock, buf, 8);
+	if(write(sock, buf, 8) < 8)
+		return AUTH_IO_ERROR;
 
 	bytesRead = read_dword(sock);
 
@@ -89,17 +103,23 @@ static int authenticate_to_floppyd(char fullauth, int sock, char *display,
 		return errcode;
 	}
 
-
-	if(bytesRead == 8) {
+	protoversion = FLOPPYD_PROTOCOL_VERSION_OLD;
+	if(bytesRead >= 12) {
 	    protoversion = read_dword(sock);
-	    read_dword(sock);
+	    cap = read_dword(sock);
 	}
-	
+
 	fprintf(stderr, "Protocol Version=%d\n", protoversion);
+	if(protoversion >= FLOPPYD_PROTOCOL_VERSION) {
+	  fprintf(stderr, "Capabilities:%s%s\n",
+		  (cap & FLOPPYD_CAP_EXPLICIT_OPEN) ? " ExplicitOpen" : "",
+		  (cap & FLOPPYD_CAP_LARGE_SEEK) ? " LargeFiles" : "");
+	}
 
 	if (fullauth) {
 		dword2byte(filelen, (Byte *) xcookie);
-		write(sock, xcookie, filelen+4);
+		if(write(sock, xcookie, filelen+4) < (ssize_t)(filelen+4))
+			return AUTH_IO_ERROR;
 
 		if (read_dword(sock) != 4) {
 			return AUTH_PACKETOVERSIZE;
@@ -116,7 +136,7 @@ static int authenticate_to_floppyd(char fullauth, int sock, char *display,
 /* ######################################################################## */
 
 static int get_host_and_port(const char* name, char** hostname, char **display,
-			     short* port)
+			     uint16_t* port)
 {
 	char* newname = strdup(name);
 	char* p;
@@ -127,10 +147,10 @@ static int get_host_and_port(const char* name, char** hostname, char **display,
 	p2 = p;
 	if (*p) p++;
 	*p2 = 0;
-	
-	*port = atoi(p);
+
+	*port = atou16(p);
 	if (*port == 0) {
-		*port = FLOPPYD_DEFAULT_PORT;	
+		*port = FLOPPYD_DEFAULT_PORT;
 	}
 
 	*display = strdup(newname);
@@ -155,57 +175,53 @@ static int get_host_and_port(const char* name, char** hostname, char **display,
 /*
  *  * Return the IP address of the specified host.
  *  */
-static IPaddr_t getipaddress(char *ipaddr)
+static in_addr_t getipaddress(char *ipaddr)
 {
-	
+
 	struct hostent  *host;
-	IPaddr_t        ip;
-	
+	in_addr_t        ip;
+
 	if (((ip = inet_addr(ipaddr)) == INADDR_NONE) &&
 	    (strcmp(ipaddr, "255.255.255.255") != 0)) {
-		
+
 		if ((host = gethostbyname(ipaddr)) != NULL) {
 			memcpy(&ip, host->h_addr, sizeof(ip));
 		}
-		
+
 		endhostent();
 	}
-	
+
 #ifdef DEBUG
-	fprintf(stderr, "IP lookup %s -> 0x%08lx\n", ipaddr, ip);
+	fprintf(stderr, "IP lookup %s -> 0x%08x\n", ipaddr, ip);
 #endif
-	  
+
 	return (ip);
 }
 
 /*
  *  * Connect to the floppyd server.
  *  */
-static int connect_to_server(IPaddr_t ip, short port)
+static int connect_to_server(in_addr_t ip, uint16_t port)
 {
-	
+
 	struct sockaddr_in      addr;
 	int                     sock;
-	
+
 	/*
 	 * Allocate a socket.
 	 */
 	if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 		return (-1);
 	}
-	
+
 	/*
 	 * Set the address to connect to.
 	 */
-	
+
 	addr.sin_family = AF_INET;
-#ifndef HAVE_HTONS
-	addr.sin_port = myhtons(port);
-#else	
 	addr.sin_port = htons(port);
-#endif	
 	addr.sin_addr.s_addr = ip;
-	
+
         /*
 	 * Connect our socket to the above address.
 	 */
@@ -218,24 +234,24 @@ static int connect_to_server(IPaddr_t ip, short port)
 	 */
 	{
 		int             on = 1;
-		setsockopt(STDIN_FILENO, SOL_SOCKET, SO_KEEPALIVE, 
+		setsockopt(STDIN_FILENO, SOL_SOCKET, SO_KEEPALIVE,
 			   (char *)&on, sizeof(on));
 
 	}
-	
+
 	return (sock);
 }
 
-int main (int argc, char** argv) 
+int main (int argc, char** argv)
 {
 	char* hostname;
 	char* display;
 	char* name;
-	short port;
+	uint16_t port;
 	int sock;
-	int reply;
+	uint32_t reply;
 	int rval;
-	int protoversion;
+	uint32_t protoversion;
 	char fullauth = 0;
 	Byte opcode = OP_CLOSE;
 
@@ -252,7 +268,7 @@ int main (int argc, char** argv)
 	}
 
 	rval = get_host_and_port(name, &hostname, &display, &port);
-	
+
 	if (!rval) return -1;
 
 	sock = connect_to_server(getipaddress(hostname), port);
@@ -263,7 +279,7 @@ int main (int argc, char** argv)
 			hostname, port);
 		return -1;
 	}
-	
+
 	protoversion = FLOPPYD_PROTOCOL_VERSION;
 	while(1) {
 	    reply = authenticate_to_floppyd(fullauth, sock, display,
@@ -279,17 +295,26 @@ int main (int argc, char** argv)
 	}
 
 	if (reply != 0) {
-		fprintf(stderr, 
+		fprintf(stderr,
 			"Connection to floppyd failed:\n"
 			"%s\n", AuthErrors[reply]);
 		return -1;
 	}
-	
+
 	free(hostname);
 	free(display);
 
-	write_dword(sock, 1);
-	write(sock, &opcode, 1);
+	if(write_dword(sock, 1) < 0) {
+		fprintf(stderr,
+			"Short write to floppyd:\n"
+			"%s\n", strerror(errno));
+	}
+
+	if(write(sock, &opcode, 1) < 0) {
+		fprintf(stderr,
+			"Short write to floppyd:\n"
+			"%s\n", strerror(errno));
+	}
 
 	close(sock);
 
