@@ -42,12 +42,18 @@ typedef struct SimpleFile_t {
 #ifdef OS_hpux
     int size_limited;
 #endif
+    const char *postcmd;
 } SimpleFile_t;
 
 
 #include "lockdev.h"
 
+#ifdef OS_mingw32msvc
+/* [s]size_t change in 32/64 build, but win read/write are always [u]int (32) */
+typedef int (*iofn) (int, void *, unsigned);
+#else
 typedef ssize_t (*iofn) (int, void *, size_t);
+#endif
 
 static ssize_t file_io(SimpleFile_t *This, char *buf,
 		       mt_off_t where, size_t len,
@@ -133,9 +139,11 @@ static int file_free(Stream_t *Stream)
 {
 	DeclareThis(SimpleFile_t);
 
-	if (This->fd > 2)
-		return close(This->fd);
-	else
+	if (This->fd > 2) {
+		int ret = close(This->fd);
+		postcmd(This->postcmd);
+		return ret;
+	} else
 		return 0;
 }
 
@@ -309,6 +317,9 @@ APIRET rc;
 			This->fd = 0;
 		else
 			This->fd = 1;
+#if defined(OS_mingw32msvc)  /* Windows defaults to text mode */
+		_setmode(This->fd, _O_BINARY);
+#endif
 		This->seekable = 0;
 		if (MT_FSTAT(This->fd, &This->statbuf) < 0) {
 		    Free(This);
@@ -334,6 +345,8 @@ APIRET rc;
 	}
 
 	precmd(dev);
+	if(dev)
+	    This->postcmd=dev->postcmd;
 	if(IS_PRIVILEGED(dev) && !(mode2 & NO_PRIV))
 		reclaim_privs();
 
@@ -440,6 +453,7 @@ APIRET rc;
 	return &This->head;
  exit_0:
 	close(This->fd);
+	postcmd(This->postcmd);
  exit_1:
 	Free(This);
 	return NULL;
